import ConfigParser, os, sys, traceback, urllib2

from packaging import Package

try:
    import PyQt4
    import PyQt4.QtGui
except:
    PyQt4 = None

# XXX ~/Library/Application Support/Oboe
if 'linux' in sys.platform:
    installloc = os.path.expanduser('~/.oboesync')

    try:
        os.makedirs(installloc)
    except:
        pass
    sys.path.append(installloc)
    os.chdir(installloc)

def getRunDir():
    try:
        sys.frozen
    except AttributeError:
        path = sys.argv[0]
    else:
        path = sys.executable

    return os.path.dirname(os.path.abspath(path))

class OboePatcher(object):
    PACKAGE = 'oboesync3'
    MAIN_FILE = 'oboe_pyqt4'
    
    def __init__(self):
        if 'win32' in sys.platform:
            os.chdir(getRunDir())
        
        self.updateClient()
    
    def displayError(self, *args):
        error = ' '.join(args)
        
        traceback.print_exc()
        
        try:
            tb = traceback.format_exc()
        except:
            tb = None
        
        if tb:
            if 'darwin' in sys.platform:
                path = os.path.expanduser('~/Library/Application Support/MP3tunes/oboe.log')
                try:
                    os.makedirs(path.rsplit('/', 1)[0])
                except:
                    pass
            else:
                path = os.path.expanduser('~/.oboesync/oboe.log')
                
                try:
                    os.makedirs(path.rsplit('/', 1)[0])
                except:
                    pass
            
            try:
                fp = file(path, 'w')
            except:
                path = os.path.expanduser('~/.oboesync/oboe.log')
                
                try:
                    os.makedirs(path.rsplit('/', 1)[0])
                except:
                    pass
                
                try:
                    fp = file(path, 'w')
                except:
                    error += '\nThe log file could not be written'
            
            fp.write(tb)
            fp.close()
        
        if PyQt4:
            app = PyQt4.QtGui.QApplication(sys.argv)
            msgbox = PyQt4.QtGui.QMessageBox(PyQt4.QtGui.QMessageBox.Critical, 'LockerSync', error)
            msgbox.show()
            app.exec_()
        else:
            print >>sys.stderr, 'Error:', error
        
        sys.exit(1)
    
    def updateClient(self):
        url = self.getUpdateURL()
        override = url.endswith('.dei')
        
        if override:
            self.getFullArchive(url)
        else:
            self.getPatchIndex(url)
        
        try:
            self.package = Package(self.PACKAGE + '.dei')
        except:
            if override:
                self.displayError('Could not open override archive')
            
            self.getFullArchive(url, patch=True)
            
            try:
                self.package = Package(self.PACKAGE + '.dei')
            except:
                self.displayError('Coult not open fullpatch archive')
        
        if not override:
            self.performUpdate(url)
        
        self.runClient()
    
    def runClient(self):
        for name in self.package.files:
            try:
                if '/' in name:
                    dirname = name.rsplit('/', 1)[0]
                    try:
                        os.makedirs(dirname)
                    except OSError:
                        pass
                file(name, 'wb').write(self.package[name])
            except:
                pass

        module = __import__(self.MAIN_FILE)
        module.main()
    
    def performUpdate(self, url):
        for name in self.updateIndex:
            if name in self.package.files.keys():
                if self.updateIndex[name][1] == self.package.sha[name]:
                    continue
            
            try:
                conn = urllib2.urlopen(self.updateIndex[name][0])
            except:
                self.displayError('Could not contact patch server to retrieve file %r' % name)
            
            try:
                data = conn.read()
            except:
                self.displayError('A network error was encountered in retrieving file %r' % name)
            
            conn.close()
            
            self.package[name] = data
        
        try:
            self.package.Save()
        except:
            self.displayError('Could not write to package file')
    
    def getPatchIndex(self, url):
        try:
            conn = urllib2.urlopen(url + '?type=' + self.PACKAGE)
        except:
            self.displayError('Could not contact patch server to retrieve timestamp')
        
        try:
            lines = conn.read().split('\n')
        except:
            self.displayError('A network error was encountered in retrieving timestamp')
        
        conn.close()
        
        self.timestamp = lines[0]
        self.updateIndex = {}
        
        for line in lines[1:]:
            if not line:
                continue
            try:
                filename, url, sha = line.split(' ')
            except:
                self.displayError('A malformed response was received from the patch server')
            
            self.updateIndex[filename] = (url, sha)
    
    def getFullArchive(self, url, patch=False):
        if patch:
            url = '%s?type=%s&v=%s&updatefile=fullpatch' % (url, self.PACKAGE, self.timestamp)
        
        try:
            conn = urllib2.urlopen(url)
        except:
            self.displayError('Could not open URL', url)
        
        try:
            fp = file(self.PACKAGE + '.dei', 'wb')
        except:
            self.displayError('Could not open archive file for writing')
        
        while True:
            try:
                data = conn.read(1024)
            except:
                self.displayError('A network error was encountered in retrieving archive')
            
            if len(data) < 1:
                break
            
            try:
                fp.write(data)
            except:
                self.displayError('An error was encountered writing the archive file')
        
        fp.close()
        conn.close()
    
    def getUpdateURL(self, retry=False):
        config = ConfigParser.RawConfigParser()
        read = config.read('servers.ini')
        
        if not 'servers.ini' in read:
            if retry:
                self.displayError('Could not read server configuration file')
            
            try:
                fp = file('servers.ini', 'w')
            except:
                self.displayError('You do not have permission to write to the directory where LockerSync is installed.')
            
            print >>fp, '[Servers]'
            print >>fp, 'Update=http://ws.mp3tunes.com/api/v0/clientUpdate'
            print >>fp, 'Login=http://shop.mp3tunes.com/'
            print >>fp, 'API=http://ws.mp3tunes.com/'
            print >>fp, 'Content=http://content.mp3tunes.com/'
            print >>fp, ''
            
            fp.close()
            
            return self.getUpdateURL(retry=True)
        
        try:
            return config.get('Servers', 'Archive')
        except:
            try:
                return config.get('Servers', 'Update')
            except:
                if retry:
                    self.displayError('Could not get update URL from server configuration')
                
                return self.getUpdateURL(retry=True)

if __name__=='__main__':
    OboePatcher()

try:
    import global_imports
except:
    pass
