import struct, sha

class PackageException(Exception):
    pass

class Package:
    def __init__(self, filename, load=True):
        self.filename = filename
        self.files = {}
        self.sha = {}
        if load:
            self.Load()
    def Load(self):
        print 'Loading package:', self.filename
        try:
            fp = file(self.filename, 'rb')
        except IOError:
            raise PackageException('Could not load package file')
        d = fp.read(4)
        if len(d) != 4:
            raise PackageException('Could not read count')
        count = struct.unpack('<L', d)[0]
        for i in range(count):
            try:
                d = fp.read(8)
            except IOError:
                raise PackageException('Could not read lengths')
            if len(d) != 8:
                raise PackageException('Malformed data')
            fnlen, flen = struct.unpack('<LL', d)

            try:
                fn = fp.read(fnlen)
            except IOError:
                raise PackageException('Could not read filename')
            if len(fn) != fnlen:
                raise PackageException('Malformed filename')
            print 'Reading file:', fn

            try:
                d = fp.read(flen)
            except IOError:
                raise PackageException('Could not read file')
            if len(d) != flen:
                raise PackageException('Malformed file data')
            self.files[fn] = d

            try:
                d = fp.read(40)
            except IOError:
                raise PackageException('Could not read SHA1')
            if len(d) != 40:
                raise PackageException('Malformed SHA1 hash')
            self.sha[fn] = d
            
        fp.close()
        print 'Loaded', len(self.files), 'files.'
    def Save(self):
        print 'Saving package:', self.filename
        fp = file(self.filename, 'wb')
        fp.write(struct.pack('<L', len(self.files)))
        for fn in self.files:
            print 'Saving file:', fn
            fp.write(struct.pack('<LL', len(fn), len(self.files[fn])))
            fp.write(fn)
            fp.write(self.files[fn])
            fp.write(self.sha[fn])
        print 'Saved', len(self.files), 'files.'
        
        fp.flush()
        fp.close()
    def __getitem__(self, name):
        if name in self.files:
            return self.files[name]
        else:
            return False
    def __setitem__(self, name, value):
        self.files[name] = value
            
        self.sha[name] = sha.new(self.files[name]).hexdigest()
        
        return value
